@@ -20,6 +20,7 @@
  */
 
 #include "gtksourcegutterrenderer.h"
+#include "gtksourcegutterrenderer-private.h"
 #include "gtksourceview-marshal.h"
 #include "gtksourceview-typebuiltins.h"
 #include "gtksourceview-i18n.h"
@@ -40,6 +41,7 @@ enum
 struct _GtkSourceGutterRendererPrivate
 {
 	GtkTextView *view;
+	GtkTextBuffer *buffer;
 	GtkTextWindowType window_type;
 
 	gint xpad;
@@ -50,7 +52,7 @@ struct _GtkSourceGutterRendererPrivate
 
 	gint size;
 
-	GdkColor background_gdk;
+	GdkRGBA background_color;
 	gboolean background_set;
 
 	GtkSourceGutterRendererAlignmentMode alignment_mode;
@@ -76,7 +78,7 @@ enum
 	PROP_ALIGNMENT_MODE,
 	PROP_WINDOW_TYPE,
 	PROP_SIZE,
-	PROP_BACKGROUND_GDK,
+	PROP_BACKGROUND_RGBA,
 	PROP_BACKGROUND_SET
 };
 
@@ -86,6 +88,62 @@ gtk_source_gutter_renderer_finalize (GObject *object)
 	G_OBJECT_CLASS (gtk_source_gutter_renderer_parent_class)->finalize (object);
 }
 
+static void
+on_buffer_changed (GtkTextView             *view,
+                   GParamSpec              *spec,
+                   GtkSourceGutterRenderer *renderer)
+{
+	GtkTextBuffer* buffer;
+
+	if (GTK_SOURCE_GUTTER_RENDERER_GET_CLASS (renderer)->change_buffer)
+	{
+		GTK_SOURCE_GUTTER_RENDERER_GET_CLASS (renderer)->change_buffer (renderer,
+		                                                                renderer->priv->buffer);
+	}
+
+	buffer = gtk_text_view_get_buffer (view);
+
+	renderer->priv->buffer = buffer;
+	g_object_add_weak_pointer (G_OBJECT (buffer), (gpointer)&renderer->priv->buffer);
+}
+
+static void
+renderer_change_view_impl (GtkSourceGutterRenderer *renderer,
+                           GtkTextView             *old_view)
+{
+	if (old_view)
+	{
+		g_signal_handlers_disconnect_by_func (old_view,
+		                                      G_CALLBACK (on_buffer_changed),
+		                                      renderer);
+	}
+
+	if (renderer->priv->view)
+	{
+		g_signal_connect (renderer->priv->view,
+		                  "notify::buffer",
+		                  G_CALLBACK (on_buffer_changed),
+		                  renderer);
+	}
+}
+
+static void
+gtk_source_gutter_renderer_dispose (GObject *object)
+{
+	GtkSourceGutterRenderer *renderer;
+
+	renderer = GTK_SOURCE_GUTTER_RENDERER (object);
+
+	if (renderer->priv->view)
+	{
+		_gtk_source_gutter_renderer_set_view (renderer,
+		                                      NULL,
+		                                      GTK_TEXT_WINDOW_PRIVATE);
+	}
+
+	G_OBJECT_CLASS (gtk_source_gutter_renderer_parent_class)->dispose (object);
+}
+
 static gboolean
 set_visible (GtkSourceGutterRenderer *renderer,
              gboolean                 visible)
@@ -217,29 +275,27 @@ set_size (GtkSourceGutterRenderer *renderer,
 }
 
 static void
-set_background_gdk_set (GtkSourceGutterRenderer *renderer,
-                        gboolean                 isset)
+set_background_color_set (GtkSourceGutterRenderer *renderer,
+                         gboolean                 isset)
 {
-	if (isset == renderer->priv->background_set)
+	if (isset != renderer->priv->background_set)
 	{
-		return;
+		renderer->priv->background_set = isset;
+		gtk_source_gutter_renderer_queue_draw (renderer);
 	}
-
-	renderer->priv->background_set = isset;
-	gtk_source_gutter_renderer_queue_draw (renderer);
 }
 
 static void
-set_background_gdk (GtkSourceGutterRenderer *renderer,
-                    const GdkColor          *color)
+set_background_color (GtkSourceGutterRenderer *renderer,
+                      const GdkRGBA          *color)
 {
 	if (!color)
 	{
-		set_background_gdk_set (renderer, FALSE);
+		set_background_color_set (renderer, FALSE);
 	}
 	else
 	{
-		renderer->priv->background_gdk = *color;
+		renderer->priv->background_color = *color;
 		renderer->priv->background_set = TRUE;
 
 		gtk_source_gutter_renderer_queue_draw (renderer);
@@ -283,13 +339,13 @@ gtk_source_gutter_renderer_set_property (GObject      *object,
 		case PROP_SIZE:
 			set_size (self, g_value_get_int (value));
 			break;
-		case PROP_BACKGROUND_GDK:
-			set_background_gdk (self,
-			                    g_value_get_boxed (value));
+		case PROP_BACKGROUND_RGBA:
+			set_background_color (self,
+			                      g_value_get_boxed (value));
 			break;
 		case PROP_BACKGROUND_SET:
-			set_background_gdk_set (self,
-			                        g_value_get_boolean (value));
+			set_background_color_set (self,
+			                          g_value_get_boolean (value));
 			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -334,8 +390,8 @@ gtk_source_gutter_renderer_get_property (GObject    *object,
 		case PROP_SIZE:
 			g_value_set_int (value, self->priv->size);
 			break;
-		case PROP_BACKGROUND_GDK:
-			g_value_set_boxed (value, &self->priv->background_gdk);
+		case PROP_BACKGROUND_RGBA:
+			g_value_set_boxed (value, &self->priv->background_color);
 			break;
 		case PROP_BACKGROUND_SET:
 			g_value_set_boolean (value, self->priv->background_set);
@@ -349,24 +405,22 @@ gtk_source_gutter_renderer_get_property (GObject    *object,
 static void
 renderer_draw_impl (GtkSourceGutterRenderer      *renderer,
                     cairo_t                      *cr,
-                    const GdkRectangle           *background_area,
-                    const GdkRectangle           *cell_area,
+                    GdkRectangle                 *background_area,
+                    GdkRectangle                 *cell_area,
                     GtkTextIter                  *start,
                     GtkTextIter                  *end,
                     GtkSourceGutterRendererState  state)
 {
-	if (!renderer->priv->background_set)
+	if (renderer->priv->background_set)
 	{
-		return;
+		cairo_save (cr);
+
+		gdk_cairo_rectangle (cr, background_area);
+		gdk_cairo_set_source_rgba (cr, &renderer->priv->background_color);
+
+		cairo_fill (cr);
+		cairo_restore (cr);
 	}
-
-	cairo_save (cr);
-
-	gdk_cairo_rectangle (cr, background_area);
-	gdk_cairo_set_source_color (cr, &renderer->priv->background_gdk);
-
-	cairo_fill (cr);
-	cairo_restore (cr);
 }
 
 static void
@@ -375,11 +429,13 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = gtk_source_gutter_renderer_finalize;
+	object_class->dispose = gtk_source_gutter_renderer_dispose;
 
 	object_class->get_property = gtk_source_gutter_renderer_get_property;
 	object_class->set_property = gtk_source_gutter_renderer_set_property;
 
 	klass->draw = renderer_draw_impl;
+	klass->change_view = renderer_change_view_impl;
 
 	g_type_class_add_private (object_class, sizeof (GtkSourceGutterRendererPrivate));
 
@@ -600,7 +656,7 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 	                                                      _("The View"),
 	                                                      _("The view"),
 	                                                      GTK_TYPE_TEXT_VIEW,
-	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	                                                      G_PARAM_READABLE));
 
 	/**
 	 * GtkSourceGutterRenderer:alignment-mode:
@@ -634,7 +690,7 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 	                                                    _("The window type"),
 	                                                    GTK_TYPE_TEXT_WINDOW_TYPE,
 	                                                    GTK_TEXT_WINDOW_PRIVATE,
-	                                                    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	                                                    G_PARAM_READABLE));
 
 	g_object_class_install_property (object_class,
 	                                 PROP_SIZE,
@@ -647,14 +703,13 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 	                                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	g_object_class_install_property (object_class,
-	                                 PROP_BACKGROUND_GDK,
-	                                 g_param_spec_boxed ("background-gdk",
-	                                                     "Background GDK",
+	                                 PROP_BACKGROUND_RGBA,
+	                                 g_param_spec_boxed ("background-rgba",
+	                                                     "Background Color",
 	                                                     "The background color",
-	                                                     GDK_TYPE_COLOR,
+	                                                     GDK_TYPE_RGBA,
 	                                                     G_PARAM_READWRITE));
 
-
 	g_object_class_install_property (object_class,
 	                                 PROP_BACKGROUND_SET,
 	                                 g_param_spec_boolean ("background-set",
@@ -687,8 +742,8 @@ gtk_source_gutter_renderer_init (GtkSourceGutterRenderer *self)
 void
 gtk_source_gutter_renderer_begin (GtkSourceGutterRenderer *renderer,
                                   cairo_t                 *cr,
-                                  const GdkRectangle      *background_area,
-                                  const GdkRectangle      *cell_area,
+                                  GdkRectangle            *background_area,
+                                  GdkRectangle            *cell_area,
                                   GtkTextIter             *start,
                                   GtkTextIter             *end)
 {
@@ -734,8 +789,8 @@ gtk_source_gutter_renderer_begin (GtkSourceGutterRenderer *renderer,
 void
 gtk_source_gutter_renderer_draw (GtkSourceGutterRenderer      *renderer,
                                  cairo_t                      *cr,
-                                 const GdkRectangle           *background_area,
-                                 const GdkRectangle           *cell_area,
+                                 GdkRectangle                 *background_area,
+                                 GdkRectangle                 *cell_area,
                                  GtkTextIter                  *start,
                                  GtkTextIter                  *end,
                                  GtkSourceGutterRendererState  state)
@@ -795,7 +850,7 @@ gtk_source_gutter_renderer_end (GtkSourceGutterRenderer *renderer)
 gboolean
 gtk_source_gutter_renderer_query_activatable (GtkSourceGutterRenderer *renderer,
                                               GtkTextIter             *iter,
-                                              const GdkRectangle      *area,
+                                              GdkRectangle            *area,
                                               GdkEvent                *event)
 {
 	gboolean ret;
@@ -832,7 +887,7 @@ gtk_source_gutter_renderer_query_activatable (GtkSourceGutterRenderer *renderer,
 void
 gtk_source_gutter_renderer_activate (GtkSourceGutterRenderer *renderer,
                                      GtkTextIter             *iter,
-                                     const GdkRectangle      *area,
+                                     GdkRectangle            *area,
                                      GdkEvent                *event)
 {
 	g_return_if_fail (GTK_IS_SOURCE_GUTTER_RENDERER (renderer));
@@ -879,7 +934,7 @@ gtk_source_gutter_renderer_queue_draw (GtkSourceGutterRenderer *renderer)
 gboolean
 gtk_source_gutter_renderer_query_tooltip (GtkSourceGutterRenderer *renderer,
                                           GtkTextIter             *iter,
-                                          const GdkRectangle      *area,
+                                          GdkRectangle            *area,
                                           gint                     x,
                                           gint                     y,
                                           GtkTooltip              *tooltip)
@@ -1187,7 +1242,7 @@ gtk_source_gutter_renderer_set_size (GtkSourceGutterRenderer *renderer,
 /**
  * gtk_source_gutter_renderer_get_background:
  * @renderer: a #GtkSourceGutterRenderer
- * @color: (out caller-allocates) (allow-none): return value for a #GdkColor
+ * @color: (out caller-allocates) (allow-none): return value for a #GdkRGBA
  *
  * Get the background color of the renderer.
  *
@@ -1196,22 +1251,22 @@ gtk_source_gutter_renderer_set_size (GtkSourceGutterRenderer *renderer,
  **/
 gboolean
 gtk_source_gutter_renderer_get_background (GtkSourceGutterRenderer *renderer,
-                                           GdkColor                *color)
+                                           GdkRGBA                 *color)
 {
 	g_return_val_if_fail (GTK_IS_SOURCE_GUTTER_RENDERER (renderer), FALSE);
 
 	if (color)
 	{
-		*color = renderer->priv->background_gdk;
+		*color = renderer->priv->background_color;
 	}
 
 	return renderer->priv->background_set;
 }
 
 /**
- * gtk_source_gutter_renderer_get_background:
+ * gtk_source_gutter_renderer_set_background:
  * @renderer: a #GtkSourceGutterRenderer
- * @color: (allow-none): a #GdkColor or %NULL
+ * @color: (allow-none): a #GdkRGBA or %NULL
  *
  * Set the background color of the renderer. If @color is set to %NULL, the
  * renderer will not have a background color.
@@ -1219,9 +1274,47 @@ gtk_source_gutter_renderer_get_background (GtkSourceGutterRenderer *renderer,
  */
 void
 gtk_source_gutter_renderer_set_background (GtkSourceGutterRenderer *renderer,
-                                           const GdkColor          *color)
+                                           const GdkRGBA           *color)
 {
 	g_return_if_fail (GTK_IS_SOURCE_GUTTER_RENDERER (renderer));
 
-	set_background_gdk (renderer, color);
+	set_background_color (renderer, color);
+}
+
+void
+_gtk_source_gutter_renderer_set_view (GtkSourceGutterRenderer *renderer,
+                                      GtkTextView             *view,
+                                      GtkTextWindowType        window_type)
+{
+	GtkTextView *old_view;
+
+	g_return_if_fail (GTK_IS_SOURCE_GUTTER_RENDERER (renderer));
+	g_return_if_fail (view == NULL || GTK_IS_TEXT_VIEW (view));
+
+	old_view = renderer->priv->view;
+
+	renderer->priv->window_type = window_type;
+
+	if (view)
+	{
+		renderer->priv->view = g_object_ref (view);
+	}
+	else
+	{
+		renderer->priv->view = NULL;
+	}
+
+	if (GTK_SOURCE_GUTTER_RENDERER_GET_CLASS (renderer)->change_view)
+	{
+		GTK_SOURCE_GUTTER_RENDERER_GET_CLASS (renderer)->change_view (renderer,
+		                                                              old_view);
+	}
+
+	if (old_view)
+	{
+		g_object_unref (old_view);
+	}
+
+	g_object_notify (G_OBJECT (renderer), "view");
+	g_object_notify (G_OBJECT (renderer), "window_type");
 }
