@@ -1,24 +1,24 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*-
- *  gtksourceview.h
+ * gtksourceview.h
+ * This file is part of GtkSourceView
  *
- *  Copyright (C) 2001 - Mikael Hermansson <tyan@linux.se> and
- *  Chris Phelps <chicane@reninet.com>
+ * Copyright (C) 2001 - Mikael Hermansson <tyan@linux.se> and
+ *                       Chris Phelps <chicane@reninet.com>
+ * Copyright (C) 2003 - Gustavo Giráldez and Paolo Maggi
  *
- *  Copyright (C) 2003 - Gustavo Giráldez and Paolo Maggi
+ * GtkSourceView is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Library General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * GtkSourceView is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef __GTK_SOURCE_VIEW_H__
@@ -29,6 +29,7 @@
 #include <gtksourceview/gtksourcebuffer.h>
 #include <gtksourceview/gtksourcecompletion.h>
 #include <gtksourceview/gtksourcegutter.h>
+#include <gtksourceview/gtksourcemarkattributes.h>
 
 G_BEGIN_DECLS
 
@@ -192,73 +193,15 @@ void 		 gtk_source_view_set_show_line_marks    (GtkSourceView   *view,
 							 gboolean         show);
 gboolean	 gtk_source_view_get_show_line_marks    (GtkSourceView   *view);
 
-#ifndef GTKSOURCEVIEW_DISABLE_DEPRECATED
-void		 gtk_source_view_set_mark_category_pixbuf
-							(GtkSourceView   *view,
-							const gchar      *category,
-							GdkPixbuf        *pixbuf) G_GNUC_DEPRECATED;
-#endif
+void             gtk_source_view_set_mark_attributes    (GtkSourceView           *view,
+                                                         const gchar             *category,
+                                                         GtkSourceMarkAttributes *attributes,
+                                                         gint                     priority);
 
-void             gtk_source_view_set_mark_category_icon_from_pixbuf
-							(GtkSourceView   *view,
-							 const gchar     *category,
-							 GdkPixbuf       *pixbuf);
-
-void             gtk_source_view_set_mark_category_icon_from_stock
-							(GtkSourceView   *view,
-							 const gchar     *category,
-							 const gchar     *stock_id);
-
-void             gtk_source_view_set_mark_category_icon_from_icon_name
-							(GtkSourceView   *view,
-							 const gchar     *category,
-							 const gchar     *name);
-
-#ifndef GTKSOURCEVIEW_DISABLE_DEPRECATED
-GdkPixbuf	*gtk_source_view_get_mark_category_pixbuf
-							(GtkSourceView   *view,
-				       			 const gchar     *category) G_GNUC_DEPRECATED;
-#endif
-
-void             gtk_source_view_set_mark_category_background
-							(GtkSourceView   *view,
-							 const gchar     *category,
-							 const GdkColor  *color);
-gboolean         gtk_source_view_get_mark_category_background
-							(GtkSourceView   *view,
-							 const gchar     *category,
-							 GdkColor        *dest);
-
-/**
- * GtkSourceViewMarkTooltipFunc:
- * @mark: the #GtkSourceMark
- * @user_data: user data pointer which was passed to gtk_source_view_set_mark_category_tooltip_func()
- *
- * Function type for setting up a tooltip for #GtkSourceMark.
- * Returns: a newly-allocated string that is going to be shown as tooltip text.
- */
-typedef gchar *  (*GtkSourceViewMarkTooltipFunc)	(GtkSourceMark	*mark,
-							 gpointer	 user_data);
-void             gtk_source_view_set_mark_category_tooltip_func
-							(GtkSourceView   *view,
-							 const gchar     *category,
-							 GtkSourceViewMarkTooltipFunc func,
-							 gpointer	  user_data,
-							 GDestroyNotify   user_data_notify);
-void		 gtk_source_view_set_mark_category_tooltip_markup_func
-							(GtkSourceView   *view,
-							const gchar     *category,
-							GtkSourceViewMarkTooltipFunc markup_func,
-							gpointer         user_data,
-							GDestroyNotify   user_data_notify);
-
-void             gtk_source_view_set_mark_category_priority
-							(GtkSourceView   *view,
-							 const gchar     *category,
-							 gint priority);
-gint		 gtk_source_view_get_mark_category_priority
-							(GtkSourceView   *view,
-				       			 const gchar     *category);
+GtkSourceMarkAttributes *
+                 gtk_source_view_get_mark_attributes    (GtkSourceView           *view,
+                                                         const gchar             *category,
+                                                         gint                    *priority);
 
 void		 gtk_source_view_set_smart_home_end	(GtkSourceView             *view,
 							 GtkSourceSmartHomeEndType  smart_he);
