@@ -1,22 +1,23 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*-
  * test-widget.c
+ * This file is part of GtkSourceView
  *
  * Copyright (C) 2001 - Mikael Hermansson <tyan@linux.se>
  * Copyright (C) 2003 - Gustavo Giráldez <gustavo.giraldez@gmx.net>
  *
- * This library is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Library General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * GtkSourceView is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * GtkSourceView is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /* If TEST_XML_MEM is defined the test program will try to detect memory
@@ -34,7 +35,7 @@
 #include <gtksourceview/gtksourcelanguagemanager.h>
 #include <gtksourceview/gtksourcestyleschememanager.h>
 #include <gtksourceview/gtksourceprintcompositor.h>
-#include <gtksourceview/gtksourceiter.h>
+#include <gtksourceview/gtksourceview-typebuiltins.h>
 #ifdef TEST_XML_MEM
 #include <libxml/xmlreader.h>
 #endif
@@ -51,10 +52,6 @@ static GtkSourceStyleScheme *style_scheme = NULL;
 #define MARK_TYPE_1      "one"
 #define MARK_TYPE_2      "two"
 
-#if !GTK_CHECK_VERSION (2, 18, 0)
-#define gtk_widget_get_visible(widget)                          (GTK_WIDGET_VISIBLE (widget))
-#endif /* GTK+ < 2.18.0 */
-
 
 /* Private prototypes -------------------------------------------------------- */
 
@@ -75,6 +72,8 @@ static void       marks_toggled_cb               (GtkAction       *action,
 						  gpointer         user_data);
 static void       margin_toggled_cb              (GtkAction       *action,
 						  gpointer         user_data);
+static void       hl_syntax_toggled_cb           (GtkAction       *action,
+						  gpointer         user_data);
 static void       hl_bracket_toggled_cb          (GtkAction       *action,
 						  gpointer         user_data);
 static void       hl_line_toggled_cb             (GtkAction       *action,
@@ -135,6 +134,9 @@ static GtkActionEntry view_action_entries[] = {
 };
 
 static GtkToggleActionEntry toggle_entries[] = {
+	{ "HlSyntax", NULL, "Highlight _Syntax", NULL,
+	  "Toggle syntax highlighting",
+	  G_CALLBACK (hl_syntax_toggled_cb), FALSE },
 	{ "HlBracket", NULL, "Highlight Matching _Bracket", NULL,
 	  "Toggle highlighting of matching bracket",
 	  G_CALLBACK (hl_bracket_toggled_cb), FALSE },
@@ -203,6 +205,7 @@ static const gchar *view_ui_description =
 "    <menu action=\"ViewMenu\">"
 "      <menuitem action=\"NewView\"/>"
 "      <separator/>"
+"      <menuitem action=\"HlSyntax\"/>"
 "      <menuitem action=\"HlBracket\"/>"
 "      <menuitem action=\"ShowNumbers\"/>"
 "      <menuitem action=\"ShowMarks\"/>"
@@ -520,6 +523,17 @@ margin_toggled_cb (GtkAction *action, gpointer user_data)
 		gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
 }
 
+static void
+hl_syntax_toggled_cb (GtkAction *action, gpointer user_data)
+{
+	GtkTextBuffer *buffer;
+	g_return_if_fail (GTK_IS_TOGGLE_ACTION (action) && GTK_IS_SOURCE_VIEW (user_data));
+	buffer = gtk_text_view_get_buffer (user_data);
+	gtk_source_buffer_set_highlight_syntax (
+		GTK_SOURCE_BUFFER (buffer),
+		gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
+}
+
 static void
 hl_bracket_toggled_cb (GtkAction *action, gpointer user_data)
 {
@@ -696,11 +710,11 @@ backward_string_cb (GtkAction *action,
 static struct {
 	char *what;
 	char *replacement;
-	GtkSourceSearchFlags flags;
+	GtkTextSearchFlags flags;
 } search_data = {
 	NULL,
 	NULL,
-	GTK_SOURCE_SEARCH_CASE_INSENSITIVE
+	GTK_TEXT_SEARCH_CASE_INSENSITIVE
 };
 
 static gboolean
@@ -708,7 +722,7 @@ search_dialog (GtkWidget            *widget,
 	       gboolean              replace,
 	       char                **what_p,
 	       char                **replacement_p,
-	       GtkSourceSearchFlags *flags_p)
+	       GtkTextSearchFlags   *flags_p)
 {
 	GtkWidget *dialog;
 	GtkEntry *entry1, *entry2;
@@ -742,7 +756,7 @@ search_dialog (GtkWidget            *widget,
 	case_sensitive = g_object_new (GTK_TYPE_CHECK_BUTTON,
 				       "visible", TRUE,
 				       "label", "Case sensitive",
-				       "active", !(search_data.flags & GTK_SOURCE_SEARCH_CASE_INSENSITIVE),
+				       "active", !(search_data.flags & GTK_TEXT_SEARCH_CASE_INSENSITIVE),
 				       NULL);
 	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
 			    GTK_WIDGET (case_sensitive), FALSE, FALSE, 0);
@@ -764,7 +778,7 @@ search_dialog (GtkWidget            *widget,
 	g_free (search_data.replacement);
 	*replacement_p = search_data.replacement = g_strdup (gtk_entry_get_text (entry2));
 	*flags_p = search_data.flags = gtk_toggle_button_get_active (case_sensitive) ?
-					0 : GTK_SOURCE_SEARCH_CASE_INSENSITIVE;
+					0 : GTK_TEXT_SEARCH_CASE_INSENSITIVE;
 
 	gtk_widget_destroy (dialog);
 	return TRUE;
@@ -777,7 +791,7 @@ do_search_replace (GtkTextView *view,
 	GtkTextBuffer *buffer = gtk_text_view_get_buffer (view);
 	GtkTextIter iter;
 	char *what, *replacement;
-	GtkSourceSearchFlags flags;
+	GtkTextSearchFlags flags;
 
 	if (!search_dialog (GTK_WIDGET (view), replace, &what, &replacement, &flags))
 		return;
@@ -790,10 +804,10 @@ do_search_replace (GtkTextView *view,
 		{
 			GtkTextIter match_start, match_end;
 
-			if (!gtk_source_iter_forward_search (&iter, what, flags,
-							     &match_start,
-							     &match_end,
-							     NULL))
+			if (!gtk_text_iter_forward_search (&iter, what, flags,
+							   &match_start,
+							   &match_end,
+							   NULL))
 			{
 				break;
 			}
@@ -809,7 +823,7 @@ do_search_replace (GtkTextView *view,
 
 		gtk_text_buffer_get_iter_at_mark (buffer, &iter, gtk_text_buffer_get_insert (buffer));
 
-		if (gtk_source_iter_forward_search (&iter, what, flags, &match_start, &match_end, NULL))
+		if (gtk_text_iter_forward_search (&iter, what, flags, &match_start, &match_end, NULL))
 		{
 			gtk_text_buffer_select_range (buffer, &match_start, &match_end);
 		}
@@ -817,7 +831,7 @@ do_search_replace (GtkTextView *view,
 		{
 			GtkTextIter insert = iter;
 			gtk_text_buffer_get_start_iter (buffer, &iter);
-			if (gtk_source_iter_forward_search (&iter, what, flags, &match_start, &match_end, &insert))
+			if (gtk_text_iter_forward_search (&iter, what, flags, &match_start, &match_end, &insert))
 				gtk_text_buffer_select_range (buffer, &match_start, &match_end);
 		}
 	}
@@ -1203,9 +1217,13 @@ line_mark_activated (GtkSourceGutter *gutter,
 	const gchar *mark_type;
 
 	if (ev->button == 1)
+	{
 		mark_type = MARK_TYPE_1;
+	}
 	else
+	{
 		mark_type = MARK_TYPE_2;
+	}
 
 	buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
 
@@ -1232,53 +1250,91 @@ line_mark_activated (GtkSourceGutter *gutter,
 	g_slist_free (mark_list);
 }
 
+static void
+bracket_matched (GtkSourceBuffer           *buffer G_GNUC_UNUSED,
+                 GtkTextIter               *iter,
+                 GtkSourceBracketMatchType  state,
+                 gpointer                  *data G_GNUC_UNUSED)
+{
+	GEnumClass *eclass;
+	GEnumValue *evalue;
+
+	g_return_if_fail (iter != NULL);
+
+	eclass = G_ENUM_CLASS (g_type_class_ref (GTK_TYPE_SOURCE_BRACKET_MATCH_TYPE));
+	evalue = g_enum_get_value (eclass, state);
+
+	g_print ("Bracket match state: '%s'\n", evalue->value_nick);
+
+	if (state == GTK_SOURCE_BRACKET_MATCH_FOUND)
+	{
+		g_print ("Matched bracket: '%c' at row: %"G_GINT32_FORMAT", col: %"G_GINT32_FORMAT"\n",
+		         gtk_text_iter_get_char (iter),
+		         gtk_text_iter_get_line (iter) + 1,
+		         gtk_text_iter_get_line_offset (iter) + 1);
+	}
+
+	g_type_class_unref (eclass);
+}
 
 /* Window creation functions -------------------------------------------------------- */
 
 static gchar *
-mark_tooltip_func (GtkSourceMark *mark,
-		   gpointer	  user_data)
+mark_tooltip_func (GtkSourceMarkAttributes *attrs,
+                   GtkSourceMark           *mark,
+                   GtkSourceView           *view)
 {
 	GtkTextBuffer *buf;
 	GtkTextIter iter;
 	gint line, column;
-	
+
 	buf = gtk_text_mark_get_buffer (GTK_TEXT_MARK (mark));
-	
+
 	gtk_text_buffer_get_iter_at_mark (buf, &iter, GTK_TEXT_MARK (mark));
 	line = gtk_text_iter_get_line (&iter) + 1;
 	column = gtk_text_iter_get_line_offset (&iter);
 
-	if (strcmp (gtk_source_mark_get_category (mark), MARK_TYPE_1) == 0)
+	if (g_strcmp0 (gtk_source_mark_get_category (mark), MARK_TYPE_1) == 0)
+	{
 		return g_strdup_printf ("Line: %d, Column: %d", line, column);
+	}
 	else
+	{
 		return g_strdup_printf ("<b>Line</b>: %d\n<i>Column</i>: %d", line, column);
+	}
 }
 
 static void
 add_source_mark_pixbufs (GtkSourceView *view)
 {
-	GdkColor color;
+	GdkRGBA color;
+	GtkSourceMarkAttributes *attrs;
 
-	gdk_color_parse ("lightgreen", &color);
-	gtk_source_view_set_mark_category_background (view, MARK_TYPE_1, &color);
-	gtk_source_view_set_mark_category_icon_from_stock (view, MARK_TYPE_1, GTK_STOCK_YES);
-	gtk_source_view_set_mark_category_priority (view, MARK_TYPE_1, 1);
-	gtk_source_view_set_mark_category_tooltip_func (view,
-							MARK_TYPE_1,
-							mark_tooltip_func,
-							NULL,
-							NULL);
+	gdk_rgba_parse (&color, "lightgreen");
 
-	gdk_color_parse ("pink", &color);
-	gtk_source_view_set_mark_category_background (view, MARK_TYPE_2, &color);
-	gtk_source_view_set_mark_category_icon_from_stock (view, MARK_TYPE_2, GTK_STOCK_NO);
-	gtk_source_view_set_mark_category_priority (view, MARK_TYPE_2, 2);
-	gtk_source_view_set_mark_category_tooltip_markup_func (view,
-							       MARK_TYPE_2,
-							       mark_tooltip_func,
-							       NULL,
-							       NULL);
+	attrs = gtk_source_mark_attributes_new ();
+	gtk_source_mark_attributes_set_background (attrs, &color);
+	gtk_source_mark_attributes_set_stock_id (attrs, GTK_STOCK_YES);
+
+	g_signal_connect (attrs,
+	                  "query-tooltip-markup",
+	                  G_CALLBACK (mark_tooltip_func),
+	                  view);
+	gtk_source_view_set_mark_attributes (view, MARK_TYPE_1, attrs, 1);
+
+	gdk_rgba_parse (&color, "pink");
+
+	attrs = gtk_source_mark_attributes_new ();
+
+	gtk_source_mark_attributes_set_background (attrs, &color);
+	gtk_source_mark_attributes_set_stock_id (attrs, GTK_STOCK_NO);
+
+	g_signal_connect (attrs,
+	                  "query-tooltip-markup",
+	                  G_CALLBACK (mark_tooltip_func),
+	                  view);
+
+	gtk_source_view_set_mark_attributes (view, MARK_TYPE_2, attrs, 2);
 }
 
 static GtkWidget *
@@ -1310,6 +1366,7 @@ create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 	g_signal_connect (buffer, "changed", G_CALLBACK (update_cursor_position), view);
 	g_signal_connect (view, "line-mark-activated", G_CALLBACK (line_mark_activated), view);
 	g_signal_connect (window, "delete-event", (GCallback) window_deleted_cb, view);
+	g_signal_connect (buffer, "bracket-matched", G_CALLBACK (bracket_matched), 0);
 
 	/* action group and UI manager */
 	action_group = gtk_action_group_new ("ViewActions");
@@ -1366,10 +1423,10 @@ create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 	font_desc = pango_font_description_from_string ("monospace");
 	if (font_desc != NULL)
 	{
-		gtk_widget_modify_font (view, font_desc);
+		gtk_widget_override_font (view, font_desc);
 		pango_font_description_free (font_desc);
 	}
-	
+
 	/* change view attributes to match those of from */
 	if (from)
 	{
@@ -1461,6 +1518,10 @@ create_main_window (GtkSourceBuffer *buffer)
 	/* retrieve the view action group at position 0 in the list */
 	action_group = g_list_nth_data (groups, 0);
 
+	action = gtk_action_group_get_action (action_group, "HlSyntax");
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+				      gtk_source_buffer_get_highlight_syntax (buffer));
+
 	action = gtk_action_group_get_action (action_group, "HlBracket");
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
 
@@ -1577,6 +1638,7 @@ main (int argc, char *argv[])
 	GtkSourceStyleSchemeManager *sm;
 	GtkSourceBuffer *buffer;
 
+	gboolean no_syntax = FALSE;
 	gchar *builtin_lang_dirs[] = {TOP_SRCDIR "/data/language-specs", NULL};
 	gchar *builtin_sm_dirs[] = {TOP_SRCDIR "/data/styles", NULL};
 	gchar **dirs;
@@ -1587,6 +1649,7 @@ main (int argc, char *argv[])
 	GOptionContext *context;
 
 	GOptionEntry entries[] = {
+	  { "no-syntax", 'n', 0, G_OPTION_ARG_NONE, &no_syntax, "Disable syntax highlighting", NULL},
 	  { "style-scheme", 's', 0, G_OPTION_ARG_STRING, &style_scheme_id, "Style scheme name to use", "SCHEME"},
 	  { "default-paths", 'd', 0, G_OPTION_ARG_NONE, &use_default_paths, "Use default search paths", NULL},
 	  { NULL }
@@ -1652,6 +1715,8 @@ main (int argc, char *argv[])
 	/* create buffer */
 	buffer = gtk_source_buffer_new (NULL);
 
+	gtk_source_buffer_set_highlight_syntax (buffer, !no_syntax);
+
 	if (argc > 1)
 		open_file (buffer, argv [1]);
 	else
